#-*- Mode: CPerl -*-

## File: Pd/Send.pm
## Author: Bryan Jurish <moocow@ling.uni-potsdam.de>
## Description:
##  + Pd netsend interface for perl
##======================================================================

package Pd::Socket::Send;
use Socket;
use IO::Socket;
use IO::Socket::INET;

@ISA = qw(IO::Socket::INET);

##======================================================================
## Generic Constructor
##======================================================================

## $sock = Pd::Socket::Send->new(%args);
##   + %args keys: PeerAddr, PeerPort, etc.
##     - see IO::Socket::INET
sub new {
  my ($that,%args) = @_;
  $args{Proto} = 'tcp' if (!exists($args{Proto}));
  $args{Type} = SOCK_STREAM if (!exists($args{Type}));
  my $self = IO::Socket::INET->new(%args);
  $self->autoflush(1) if ($self); ##-- this should be the default
  return defined($self) ? bless($self, ref($that) || $that) : undef;
}

## undef = $sock->pdsend(@msg);
##   + send semicolon-terminated @msg to Pd over the socket
sub pdsend {
  my $self = shift;
  return $self->print(@_, ";\n\r");
}


##======================================================================
## Pd::Socket::Send::TCP
##======================================================================

package Pd::Socket::Send::TCP;
use Socket;
use IO::Socket;
use IO::Socket::INET;
@ISA = qw(Pd::Socket::Send);

## $sock = Pd::Socket::Send::TCP->new(%args);
##   + %args keys:
##     - see IO::Socket::INET
sub new {
  my $that = shift;
  return Pd::Socket::Send::new($that,
			       @_,
			       Proto=>'tcp',
			       Type=>SOCK_STREAM);
}

##======================================================================
## Pd::Socket::Send::UDP
##======================================================================

package Pd::Socket::Send::UDP;
use Socket;
use IO::Socket;
use IO::Socket::INET;
@ISA = qw(Pd::Socket::Send);

## $sock = Pd::Socket::Send::UDP->new(%args);
##   + %args keys:
##     - see IO::Socket::INET
sub new {
  my $that = shift;
  return Pd::Socket::Send::new($that,
			       @_,
			       Proto=>'udp',
			       Type=>SOCK_DGRAM);
}



##======================================================================
## Docs
=pod

=head1 NAME

Pd::Socket::Send - Perl interface to Pd [netsend].

=head1 SYNOPSIS

 use Pd::Socket::Send;

 # Create a socket...
 $tcp_sock = Pd::Socket::Send::TCP->new(PeerAddr=>'localhost:3000');
 $udp_sock = Pd::Socket::Send::UDP->new(PeerAddr=>'localhost:3001'); # broken!

 # ... and send some data
 $tcp_sock->print("list bla bla bla;\n\r");
 $udp_sock->print("list foo bar baz;\n\r");  # broken!

 # (try the above with the Pd [netreceive] help patch open)

=cut

##======================================================================
## Description
=pod

=head1 DESCRIPTION

Pd::Socket::Send provides a Perl API similar to the PureData
[netsend] object.

=cut

##======================================================================
## Footer
=pod

=head1 ACKNOWLEDGEMENTS

perl by Larry Wall.

Pd by Miller Puckette and others.

=head1 KNOWN BUGS

Pd::Socket::Send::UDP appears to be broken.

=head1 AUTHOR

Bryan Jurish E<lt>jurish@ling.uni-potsdam.deE<gt>

=head1 COPYRIGHT

Copyright (c) 2004, Bryan Jurish.  All rights reserved.

This package is free software.  You may redistribute it
and/or modify it under the same terms as Perl itself.

=head1 SEE ALSO

perl(1),
pd(1),
pdsend(1),
pdreceive(1),
Pd::Socket(3pm),
Pd::Socket::Receive(3pm).

=cut
